#!/usr/local/bin/python

import sys
import os
import os.path
from optparse import OptionParser
from abac.cred_util import make_abac_ID, call_creddy_attribute

class option_parser(OptionParser):
    def __init__(self):
	OptionParser.__init__(self)
	self.add_option('--directory', dest='dir', default='~/.gcf',
		help='Directory populated by gen-certs.py')
	self.add_option('--abac-dir', dest='abac_dir', default='abac',
		help='The abac subdirectory to generate')
	self.add_option('--ch-prefix', dest='ch', default='ch',
		help='prefix for clearinghouse certs')
	self.add_option('--am-prefix', dest='am', default='am',
		help='prefix for aggregate manager certs')
	self.add_option('--user-prefix', dest='user', default='alice',
		help='prefix for user certs')

def make_attr_cert(keyfile, certfile, role, subject, dest):
    cert = call_creddy_attribute(keyfile, certfile, role, 365, 
	    subject_cert=subject)
    try:
	f = open(dest, 'w')
	print >>f, cert,
	f.close()
    except EnvironmentError, e:
	warn("Could not create %s: %s" % (e.filename, e.strerror))



opts, args = option_parser().parse_args()

# Make sure we can access the ABAC directory
dir = os.path.expanduser(opts.dir)
abac_dir = os.path.join(dir, opts.abac_dir)
if not os.access(abac_dir, os.F_OK):
    os.mkdir(abac_dir, 0700)
elif not os.path.isdir(abac_dir):
    sys.exit('%s is not a directory' % abac_dir)
elif not os.access(abac_dir, os.W_OK):
    sys.exit('%s is not writeable' % abac_dir)


am = opts.am
ch = opts.ch
user=opts.user

# Translate the AM and CH certificates into ABAC IDs
for c in (user, am, ch):
    keyfile = os.path.join(dir, '%s-key.pem' % c)
    certfile = os.path.join(dir, '%s-cert.pem' % c)
    destfile = '%s-cert.pem' % c
    try:
	make_abac_ID(certfile, keyfile, abac_dir, destfile)
    except EnvironmentError, e:
	print >>sys.stderr, "Could not access %s: %s" % (e.filename, e.strerror)

# add the AM.list attribute to the user
keyfile = os.path.join(dir, '%s-key.pem' % am)
certfile = os.path.join(abac_dir, '%s-cert.pem' % am)
subject = os.path.join(abac_dir, '%s-cert.pem' % user)
make_attr_cert(keyfile, certfile, 'list', subject, 
	os.path.join(abac_dir, '%s-list.der' % user))

# add the AM.clearinghouse attribute to the CH.
subject = os.path.join(abac_dir, '%s-cert.pem' % ch)
make_attr_cert(keyfile, certfile, 'clearinghouse', subject, 
	os.path.join(abac_dir, '%s-clearinghouse.der' % ch))
